<?php
declare(strict_types=1);

namespace Inti\Wp\Middleware;

use Doctrine\DBAL\Connection;
use Inti\Wp\Plugin\Service\PluginRunnerInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;

use function add_action;
use function define;
use function dirname;
use function rtrim;

/**
 * Class Bootstrap
 *
 * @package Inti\Wp\Middleware
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-middleware
 */
final class Bootstrap implements MiddlewareInterface
{
    private Connection $conn;
    private PluginRunnerInterface $runner;
    private array $config;

    public function __construct(Connection $conn, PluginRunnerInterface $runner, array $config)
    {
        $this->conn = $conn;
        $this->runner = $runner;
        $this->config = $config;
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        /** @noinspection PhpUnusedLocalVariableInspection */
        $table_prefix = 'wp_';

        /** @noinspection PhpInternalEntityUsedInspection */
        $dbParams = $this->conn->getParams();

        define('DB_NAME', $dbParams['dbname']);
        define('DB_USER', $dbParams['user']);
        define('DB_PASSWORD', $dbParams['password']);
        define('DB_HOST', $dbParams['host']);
        define('DB_CHARSET', 'utf8mb4');
        define('DB_COLLATE', 'utf8mb4_croatian_ci');

        // @see https://api.wordpress.org/secret-key/1.1/salt/
        define('AUTH_KEY',         'n]57N$[g5#.6ByTu$9|u2V],pY@04ZjZ6*DMA(m5Y}RWDGF9Q -N-{,+j|xKTo +');
        define('SECURE_AUTH_KEY',  '2pZtASr{:F!-0*]LKj<+&E^->R2`-a.#EgRLj=Y@Iv<9k#h$K=toO,Q$FuYD9{mE');
        define('LOGGED_IN_KEY',    'Lr~p,|m<)K`7ideXAC(^yja?rOp21&D+5U {FDL&x]}ym_c?I8DK,AS_<25rz9j!');
        define('NONCE_KEY',        'cnBK(pIR7*~eXI1,0pDx 3>{4hk1F `TJ0l/DhwR/M{QqQQW5!)7S9r&20=N w#j');
        define('AUTH_SALT',        'iE()-G tXSTbqr.C2=xP^I&v|rw%L3j=_EEC-`l]q%kdmM%s32*4Hqr<`zopKdZj');
        define('SECURE_AUTH_SALT', 'MC=SkTJHg7 7)i@4cg{!!s6&E/`ZW@wcj7>/kE?p7K@zho{mA#erjxGXXpj& A*o');
        define('LOGGED_IN_SALT',   'F JCgF-HvE%9jZK@i!G-C-P|-.5oTQ|_fw{KO~EZt]_ASe1GOr2Gd-A{|Z>;b#59');
        define('NONCE_SALT',       '.hYQ6+$^-R@(6TwHeo+R!<?9e(`,u<Rk),@~g#8~Dt5i4Z1~+Y%$F[K]!Y2bwtlE');

        $uri = $request->getUri();
        define('WP_HOME', rtrim((string) $uri->withQuery('')->withPath('/'), '/'));
        define('WP_SITEURL', WP_HOME . '/' . basename(ABSPATH));
        define('WP_CONTENT_DIR', basename(ABSPATH) === 'wp' ? dirname(ABSPATH) : ABSPATH . 'wp-content');
        define('WP_CONTENT_URL', basename(ABSPATH) === 'wp' ? WP_HOME : WP_SITEURL . '/wp-content');

        require_once ABSPATH . 'wp-includes/plugin.php';
        add_action('plugins_loaded', [$this, 'run_plugins']);

        foreach ($this->config['globals'] ?? [] as $global) {
            global $$global;
        }

        require_once ABSPATH . 'wp-settings.php';

        return $handler->handle($request);
    }

    public function run_plugins(): void
    {
        $plugins = $this->config['plugins'] ?? [];
        $this->runner->run($plugins);
    }
}
