<?php
declare(strict_types=1);

namespace IntiTest\Wp\Middleware\Handler\Admin;

use IntiTest\Wp\Middleware\WordPressTestCase;
use Laminas\Diactoros\Uri;
use PHPUnit\Framework\TestCase;

use function is_dir;
use function rmdir;
use function scandir;
use function unlink;

/**
 * Class InstallTest
 *
 * @package IntiTest\Wp\Middleware\Handler\Admin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp-middleware
 */
final class InstallTest extends TestCase
{
    use WordPressTestCase;

    protected function setUp(): void
    {
        parent::setUp();

        $this->setUpContainer();
        $this->setUpWordPress();

        if (is_dir(ABSPATH . 'wp-content/languages')) {
            foreach (scandir(ABSPATH . 'wp-content/languages') as $file) {
                $path = ABSPATH . 'wp-content/languages/' . $file;

                if (!is_dir($path)) {
                    @unlink($path);
                }
            }

            @rmdir(ABSPATH . 'wp-content/languages');
        }
    }

    public function testWillInstallTranslationsAndRedirectForStep0()
    {
        $this->assertDirectoryDoesNotExist(ABSPATH . 'wp-content/languages');

        $response = $this->runApplication([], '/wp/wp-admin/install.php');

        $this->assertDirectoryExists(WP_CONTENT_DIR . '/languages');
        $this->assertFileExists(WP_CONTENT_DIR . '/languages/hr.po');

        $this->assertEquals(302, $response->getStatusCode());

        $redirectUri = new Uri($response->getHeaderLine('Location'));
        $this->assertEquals('language=hr&step=1', $redirectUri->getQuery());
    }

    public function testWillReturnEmptyResponseForOtherSteps()
    {
        $response = $this->runApplication([], '/wp/wp-admin/install.php?language=hr&step=1');

        $this->assertEquals(200, $response->getStatusCode());
        $this->assertEmpty($response->getHeaders());
        $this->assertEmpty($response->getBody()->getContents());
    }
}
